package chaincode

import (
	"github.com/hyperledger/fabric/core/chaincode/shim"
	pb "github.com/hyperledger/fabric/protos/peer"
	"encoding/json"
	"github.com/pkg/errors"
	"github.com/stretchr/testify/mock"
	"time"
)

type ManagerImplMock struct {
	mock.Mock
}

func (mock ManagerImplMock) GetTime() int64 {
	return int64(time.Now().Second())
}

func (mock ManagerImplMock) SetAdapter(new StubInterfaceAdapter) {
	// nothing
}

func (mock ManagerImplMock) SetTimeGetter(new TimeGetter) {
	// nothing
}

func (mock ManagerImplMock) GetAdapter() StubInterfaceAdapter {
	mockArgs := mock.Called()
	if mockArgs.Get(0) == nil {
		return nil
	}
	return mockArgs.Get(0).(StubInterfaceAdapter)
}

func (mock ManagerImplMock) GetTimeGetter() TimeGetter {
	mockArgs := mock.Called()
	if mockArgs.Get(0) == nil {
		return nil
	}
	return mockArgs.Get(0).(TimeGetter)
}

func (mock ManagerImplMock) GetState(key string) ([]byte, error) {
	mockArgs := mock.Called(key)
	if mockArgs.Get(0) == nil && mockArgs.Error(1) == nil {
		return nil, nil
	}
	if mockArgs.Get(0) != nil && mockArgs.Error(1) == nil {
		return nil, mockArgs.Error(1)
	}
	return mockArgs.Get(0).([]byte), mockArgs.Error(1)
}

func (mock ManagerImplMock) PutState(key string, value []byte) error {
	mockArgs := mock.Called(key, value)
	return mockArgs.Error(0)
}

func (mock ManagerImplMock) DelState(key string) error {
	mockArgs := mock.Called(key)
	return mockArgs.Error(0)
}

// TODO REMOVE >>> DEPRECATED <<<
func (mock ManagerImplMock) GetQueryResult(query string) (shim.StateQueryIteratorInterface, error) {
	mockArgs := mock.Called(query)
	return mockArgs.Get(0).(shim.StateQueryIteratorInterface), mockArgs.Error(1)
}

func (mock ManagerImplMock) GetStateByPartialCompositeKey(objectType string, keys []string) (shim.StateQueryIteratorInterface, error) {
	mockArgs := mock.Called(objectType, keys)
	return mockArgs.Get(0).(shim.StateQueryIteratorInterface), mockArgs.Error(1)
}

func (mock ManagerImplMock) GetObjectListByPartialCompositeKey(expectedValidObjectType ValidObject, keys []string) ([]ValidObject, error) {
	mockArgs := mock.Called(expectedValidObjectType, keys)
	return mockArgs.Get(0).([]ValidObject), mockArgs.Error(1)
}

func (mock ManagerImplMock) CreateCompositeKey(objectType string, attributes []string) (string, error) {
	mockArgs := mock.Called(objectType, attributes)
	return mockArgs.Get(0).(string), mockArgs.Error(1)
}

func (mock ManagerImplMock) SplitCompositeKey(compositeKey string) (string, []string, error) {
	mockArgs := mock.Called(compositeKey)
	return mockArgs.Get(0).(string), mockArgs.Get(1).([]string), mockArgs.Error(2)
}

func (mock ManagerImplMock) InvokeChaincode(chaincodeName string, args [][]byte, channel string) pb.Response {
	mockArgs := mock.Called(chaincodeName, args, channel)
	if mockArgs.Get(0) == nil {
		return pb.Response{Status: 404}
	}
	return mockArgs.Get(0).(pb.Response)
}

func (mock ManagerImplMock) GetTxID() string {
	mockArgs := mock.Called()
	return mockArgs.Get(0).(string)
}

func (mock ManagerImplMock) Save(obj ValidObject) (ValidObject, error) {
	mockArgs := mock.Called(obj)
	if mockArgs.Get(0) == nil && mockArgs.Error(1) == nil {
		return nil, nil
	}
	return mockArgs.Get(0).(ValidObject), mockArgs.Error(1)
}

func (mock ManagerImplMock) IsExists(obj ValidObject) (bool, error) {
	mockArgs := mock.Called(obj)
	if mockArgs.Get(0) == nil && mockArgs.Error(1) == nil {
		return false, nil
	}
	return mockArgs.Get(0).(bool), mockArgs.Error(1)
}

func (mock ManagerImplMock) GetIfExists(obj ValidObject) (bool, error) {
	mockArgs := mock.Called(obj)
	if mockArgs.Get(0) == nil && mockArgs.Error(1) == nil {
		return false, nil
	}
	return mockArgs.Get(0).(bool), mockArgs.Error(1)
}

func (mock ManagerImplMock) AddObject(obj ValidObject) (ValidObject, error) {
	mockArgs := mock.Called(obj)
	if mockArgs.Get(0) == nil && mockArgs.Error(1) == nil {
		return nil, nil
	}
	if mockArgs.Get(0) == nil && mockArgs.Error(1) != nil {
		return nil, mockArgs.Error(1)
	}
	return mockArgs.Get(0).(ValidObject), mockArgs.Error(1)
}

func (mock ManagerImplMock) UpdateObject(obj ValidObject) (ValidObject, error) {
	mockArgs := mock.Called(obj)
	if mockArgs.Get(0) == nil && mockArgs.Error(1) == nil {
		return nil, nil
	}
	if mockArgs.Get(0) == nil && mockArgs.Error(1) != nil {
		return nil, mockArgs.Error(1)
	}
	return mockArgs.Get(0).(ValidObject), mockArgs.Error(1)
}

func (mock ManagerImplMock) GetObject(obj ValidObject, keys ...string) (ValidObject, error) {
	mockArgs := mock.Called(obj, keys)
	if mockArgs.Get(0) == nil && mockArgs.Error(1) == nil {
		return nil, nil
	}
	if mockArgs.Get(0) == nil && mockArgs.Error(1) != nil {
		return nil, mockArgs.Error(1)
	}
	return mockArgs.Get(0).(ValidObject), mockArgs.Error(1)
}

func (mock ManagerImplMock) GetObjectByKeys(expectedObject ValidObject, keys []string) (ValidObject, error) {
	key, err := GetKeyByArray(keys)
	if err != nil {
		return nil, err
	}

	bytes, err := mock.GetState(key)
	if err != nil {
		return nil, errors.Errorf("GetState by key %s. Error: %s", key, err)
	}

	if bytes == nil || len(bytes) == 0 {
		return nil, errors.New("NOT_FOUND")
	}

	err = json.Unmarshal(bytes, expectedObject)
	if err != nil {
		return nil, err
	}
	return expectedObject, nil
}

func (mock ManagerImplMock) SignedInvokeChaincode(channelName string, chaincodeName string, functionName string, arg interface{}) pb.Response {
	mockArgs := mock.Called(channelName, chaincodeName, functionName, arg)
	if mockArgs.Get(0) == nil {
		return pb.Response{Status:404}
	}
	return mockArgs.Get(0).(pb.Response)
}


func (mock ManagerImplMock) PrepareInvokeChaincodeParam(channelName string, chaincodeName string, functionName string, arg interface{}) ([][]byte, error) {
	mockArgs := mock.Called(channelName, chaincodeName, functionName, arg)
	if mockArgs.Get(0) == nil && mockArgs.Error(1) == nil {
		return nil, nil
	}
	if mockArgs.Get(0) != nil && mockArgs.Error(1) == nil {
		return nil, mockArgs.Error(1)
	}
	return mockArgs.Get(0).([][]byte), mockArgs.Error(1)
}

func (mock ManagerImplMock) InvokeChaincode_GetUser(UserID string) pb.Response {
	mockArgs := mock.Called(UserID)
	if mockArgs.Get(0) == nil {
		return pb.Response{Status:404}
	}
	return mockArgs.Get(0).(pb.Response)
}

func (mock ManagerImplMock) InvokeChaincode_VerifyUserSignature(arg interface{}) pb.Response {
	mockArgs := mock.Called(arg)
	if mockArgs.Get(0) == nil {
		return pb.Response{Status:404}
	}
	return mockArgs.Get(0).(pb.Response)
}
