package chaincode

import (
	"github.com/hyperledger/fabric/core/chaincode/shim"
	pb "github.com/hyperledger/fabric/protos/peer"
	"fmt"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
)

// ChaincodeStubInterfaceAdapter stub interface to encapsulate only
// relevant parts of the shim.ChaincodeStubInterface API allowing to mock
// this functionality
type StubInterfaceAdapter interface {
	// GetState returns the value of the specified `key` from the
	// ledger. Note that GetState doesn't read data from the writeset, which
	// has not been committed to the ledger. In other words, GetState doesn't
	// consider data modified by PutState that has not been committed.
	// If the key does not exist in the state database, (nil, nil) is returned.
	GetState(key string) ([]byte, error)

	// PutState puts the specified `key` and `value` into the transaction's
	// writeset as a data-write proposal. PutState doesn't effect the ledger
	// until the transaction is validated and successfully committed.
	// Simple keys must not be an empty string and must not start with null
	// character (0x00), in order to avoid range query collisions with
	// composite keys, which internally get prefixed with 0x00 as composite
	// key namespace.
	PutState(key string, value []byte) error

	// DelState records the specified `key` to be deleted in the writeset of
	// the transaction proposal. The `key` and its value will be deleted from
	// the ledger when the transaction is validated and successfully committed.
	DelState(key string) error

	// GetQueryResult enables to run ricj queries over choudb state db
	// needed to being able effectively extract account information for
	// user, based only on partial parameters available
	// TODO REMOVE >>> DEPRECATED <<<
	GetQueryResult(query string) (shim.StateQueryIteratorInterface, error)

	// InvokeChaincode locally calls the specified chaincode `Invoke` using the
	// same transaction context; that is, chaincode calling chaincode doesn't
	// create a new transaction message.
	// If the called chaincode is on the same channel, it simply adds the called
	// chaincode read set and write set to the calling transaction.
	// If the called chaincode is on a different channel,
	// only the Response is returned to the calling chaincode; any PutState calls
	// from the called chaincode will not have any effect on the ledger; that is,
	// the called chaincode on a different channel will not have its read set
	// and write set applied to the transaction. Only the calling chaincode's
	// read set and write set will be applied to the transaction. Effectively
	// the called chaincode on a different channel is a `Query`, which does not
	// participate in state validation checks in subsequent commit phase.
	// If `channel` is empty, the caller's channel is assumed.
	InvokeChaincode(chaincodeName string, args [][]byte, channel string) pb.Response

	GetTxID() string

	// CreateCompositeKey combines the given `attributes` to form a composite
	// key. The objectType and attributes are expected to have only valid utf8
	// strings and should not contain U+0000 (nil byte) and U+10FFFF
	// (biggest and unallocated code point).
	// The resulting composite key can be used as the key in PutState().
	CreateCompositeKey(objectType string, attributes []string) (string, error)

	// SplitCompositeKey splits the specified key into attributes on which the
	// composite key was formed. Composite keys found during range queries
	// or partial composite key queries can therefore be split into their
	// composite parts.
	SplitCompositeKey(compositeKey string) (string, []string, error)

	// GetStateByPartialCompositeKey queries the state in the ledger based on
	// a given partial composite key. This function returns an iterator
	// which can be used to iterate over all composite keys whose prefix matches
	// the given partial composite key. The `objectType` and attributes are
	// expected to have only valid utf8 strings and should not contain
	// U+0000 (nil byte) and U+10FFFF (biggest and unallocated code point).
	// See related functions SplitCompositeKey and CreateCompositeKey.
	// Call Close() on the returned StateQueryIteratorInterface object when done.
	// The query is re-executed during validation phase to ensure result set
	// has not changed since transaction endorsement (phantom reads detected).
	GetStateByPartialCompositeKey(objectType string, keys []string) (shim.StateQueryIteratorInterface, error)
}

type ChaincodeParameter struct {
	ChaincodeName         string `json:"chaincode_name,omitempty"`
	ChaincodeSourceName   string `json:"chaincode_source_name,omitempty"`
	ChaincodeSourceBranch string `json:"chaincode_source_branch,omitempty"`
	RevisionNumber        string `json:"revision_number,omitempty"`
	ChannelName           string `json:"channel_name,omitempty"`
	ChaincodeVersion      string `json:"chaincode_version,omitempty"`
	Fields                Fields `json:"map,omitempty"`
}

func (chaincodeParameter ChaincodeParameter) ToString() string {
	return fmt.Sprintf("ChaincodeParameter{"+
		"chaincode_name: %s,"+
		"chaincode_source_name: %s,"+
		"chaincode_source_branch: %s,"+
		"revision_number: %s, "+
		"channel_name: %s, "+
		"chaincode_version: %s, "+
		"map: %s"+
		"}",
		chaincodeParameter.ChaincodeName,
		chaincodeParameter.ChaincodeSourceName,
		chaincodeParameter.ChaincodeSourceBranch,
		chaincodeParameter.RevisionNumber,
		chaincodeParameter.ChannelName,
		chaincodeParameter.ChaincodeVersion,
		chaincodeParameter.Fields)
}

type CommonChaincodeAction func(userParameter *requests.Parameter, systemParameter *requests.Parameter, manager IManager) (interface{}, error)

type TimeGetter func() (int64, error)

// UserChaincode chaincode capable to operate and manager
// users records, namely add, get and update user records
type CommonChaincode struct {
	Actions map[string]CommonChaincodeAction
}

type Fields map[string]string
